use utf8;
package TaskPipe::SchemaTemplate_Project_SP500;

use strict;
use warnings;

use base 'DBIx::Class::Schema';

__PACKAGE__->load_namespaces;

=head1 NAME

TaskPipe::SchemaTemplate_Project_SP500 - schema template for the SP500 sample project

=head1 DESCRIPTION

Schema Template for the SP500 sample project

=head1 AUTHOR

Tom Gracey <tomgracey@gmail.com>

=head1 COPYRIGHT AND LICENSE

Copyright (c) Tom Gracey 2018

TaskPipe is free software, licensed under

    The GNU Public License Version 3

=cut

1;
