package TaskPipe;

use strict;
use warnings;

our $VERSION = 0.08;

1;

=head1 NAME

TaskPipe - A task management framework for building web scrapers and crawlers

=head1 DESCRIPTION

This module is simply a placeholder for the TaskPipe package namespace, and does nothing on its own. For TaskPipe documentation see L<TaskPipe::Manual::Overview>. For installation see L<TaskPipe::Manual::Installation>. For help using taskpipe, type

    taskpipe help

on the command line after installation.

=cut

=head1 AUTHOR

Tom Gracey <tomgracey@gmail.com>

=head1 COPYRIGHT AND LICENSE

Copyright (c) Tom Gracey 2018

TaskPipe is free software, licensed under

    The GNU Public License Version 3
